// SPDX-License-Identifier: MIT
pragma solidity ^0.8.0;

contract SimpleToken {
    // Ім'я токена
    string public name = "SimpleToken";
    // Символ токена
    string public symbol = "STK";
    // Десяткові знаки
    uint8 public decimals = 18;
    // Загальна кількість токенів
    uint256 public totalSupply;

    // Баланси користувачів
    mapping(address => uint256) public balanceOf;
    // Дозволені витрати (allowances)
    mapping(address => mapping(address => uint256)) public allowance;

    // Події для передавання та затвердження
    event Transfer(address indexed from, address indexed to, uint256 value);
    event Approval(address indexed owner, address indexed spender, uint256 value);

    // Конструктор для створення початкової кількості токенів
    constructor(uint256 _initialSupply) {
        totalSupply = _initialSupply * 10 ** uint256(decimals);
        balanceOf[msg.sender] = totalSupply;
        emit Transfer(address(0), msg.sender, totalSupply);
    }

    /**
     * @dev Переводить `_value` токенів з повідомника на адресу `_to`
     * @param _to Адреса отримувача
     * @param _value Кількість токенів для переведення
     */
    function transfer(address _to, uint256 _value) public returns (bool success) {
        require(_to != address(0), "Invalid address");
        require(balanceOf[msg.sender] >= _value, "Insufficient balance");

        // Зменшуємо баланс відправника та збільшуємо баланс отримувача
        balanceOf[msg.sender] -= _value;
        balanceOf[_to] += _value;

        // Викликаємо подію Transfer
        emit Transfer(msg.sender, _to, _value);
        return true;
    }

    /**
     * @dev Затверджує `_spender` на витрати до `_value` токенів від власника
     * @param _spender Адреса, яка буде витрачати токени
     * @param _value Максимальна кількість токенів для витрати
     */
    function approve(address _spender, uint256 _value) public returns (bool success) {
        require(_spender != address(0), "Invalid address");

        allowance[msg.sender][_spender] = _value;
        emit Approval(msg.sender, _spender, _value);
        return true;
    }

    /**
     * @dev Переводить `_value` токенів від `_from` до `_to`, використовуючи дозволені витрати
     * @param _from Адреса відправника
     * @param _to Адреса отримувача
     * @param _value Кількість токенів для переведення
     */
    function transferFrom(address _from, address _to, uint256 _value) public returns (bool success) {
        require(_from != address(0), "Invalid from address");
        require(_to != address(0), "Invalid to address");
        require(balanceOf[_from] >= _value, "Insufficient balance");
        require(allowance[_from][msg.sender] >= _value, "Allowance exceeded");

        // Зменшуємо баланс відправника та збільшуємо баланс отримувача
        balanceOf[_from] -= _value;
        balanceOf[_to] += _value;

        // Зменшуємо дозволені витрати
        allowance[_from][msg.sender] -= _value;

        // Викликаємо подію Transfer
        emit Transfer(_from, _to, _value);
        return true;
    }
}
